/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */
/*
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#include "lib-lib.h"

rx_t *rx_compile(const char *s, int flags)
{
    rx_t *pp = p_new(rx_t, 1);

    pp->pattern = m_strdup(s);
    pp->rx = p_new(regex_t, 1);

    if (REGCOMP(pp->rx, NONULL(s), flags) != 0) {
        rx_delete(&pp);
    }

    return pp;
}

void rx_set_template(rx_t *rx, const char *tpl)
{
    const char *p = tpl;

    m_strreplace(&rx->template, tpl);
    rx->nmatch = 0;

    while ((p = strchr(p, '%'))) {
        if (isdigit(*++p)) {
            int n = strtol(p, (char **)&p, 10);
            rx->nmatch = MAX(n, rx->nmatch);
        } else {
            if (*p == '%')
                p++;
        }
    }

    rx->nmatch++;     /* match 0 is always the whole expr */
}

void rx_delete(rx_t **p)
{
    p_delete(&(*p)->pattern);
    regfree((*p)->rx);
    p_delete(&(*p)->rx);
    p_delete(&(*p)->template);
    p_delete(p);
}

int rx_list_match(rx_t *l, const char *pat)
{
    if (!pat || !*pat)
        return 0;

    while (l) {
        if (!REGEXEC(l->rx, pat))
            return 1;
        l = l->next;
    }

    return 0;
}

rx_t **rx_lookup(rx_t **l, const char *pat)
{
    if (!pat || !*pat)
        return NULL;

    while (*l) {
        if (!strcmp((*l)->pattern, pat))
            return l;
        l = &(*l)->next;
    }

    return NULL;
}

int rx_sanitize_string(char *dst, ssize_t n, const char *src)
{
    while (*src) {
        if (n <= 1)
            break;

        /* these characters must be escaped in regular expressions */
        if (strchr("^.[$()|*+?{\\", *src)) {
            if (n <= 2)
                break;

            *dst++ = '\\';
            n--;
        }

        *dst++ = *src++;
        n--;
    }

    *dst = '\0';

    return *src ? -1 : 0;
}
